! ---------------------------------------------------------------------------
!  Fdecon.f90 - Modified by Alejandro Valenciano based on
!                         Quadrature.f90 by Gabriel Alvarez 
!           Sept 5  2002.
! ---------------------------------------------------------------------------
!       Usage: Fdecon.x
!  Parameters:
!            filt ......... vector to deconvolve with (source wavefield)
!            data ......... vector to be deconvolve   (receiver wavefield)
!            decon......... vector with the result of the deconvolution (reflection strength)
!
! Description: Performs deconvolution in the Fourier domain 
! ---------------------------------------------------------------------------
program Fdecon 
	use sep

	implicit none

	integer					:: n1, n2, esize
	real					:: d1,d2,eps,scale
	real, dimension(:,:), allocatable	:: data,filt
	complex, dimension(:,:), allocatable	:: decon 

	!! hook up SEP init stuff and get input parameters from history file
	call sep_init ()
	call from_history ("n1",n1)
	call from_history ("n2",n2,1)
	call from_history ("d1",d1)
	call from_history ("d2",d2,1.0)

	!! hook up parameters from command line 
	call from_param( 'eps', eps, 1.0)
	call from_param( 'scale', scale, 10000000.)

	!! allocate working space 
	allocate (decon(n1,n2),data(n1,n2),filt(n1,n2))                                   
	call sreed ("in",data,4*n1*n2) 
	call sreed ("filt",filt,4*n1*n2) 

	!! compute decon 
	call compute_decon (n1,n2,data,filt,decon,eps,scale)            

	!! output parameters to history file
	esize=8
	call to_history ("n1", n1)
	call to_history ("n2", n2)
	call to_history ("n3", 1)
	call to_history ("esize",esize)
	call to_history ("o1",0)
	call to_history ("o2",0)
	call to_history ("d1",d1)
	call to_history ("d2",d2)
	call sep_close()

	!! output decon 
	call srite ("out",decon,esize*n1*n2) 

	!! free allocated space
	deallocate (decon,data,filt)

	call exit (0)

end program Fdecon 

subroutine compute_decon (nt,nx,data,filt,decon,eps,scale)
        use sep
        implicit none 
	!! subroutine arguments
	integer				 :: nt,nx
        real                             :: eps, scale
	real,dimension(nt,nx)		 :: data,filt
	complex,dimension(nt,nx)	 :: decon 

	!! internal variables
	integer				 :: it,ix,ntpad,cont,jpad2
	real                             :: ave 
	complex,dimension(:),allocatable :: ctemp,ctemp1,ctemp2

	!! compute next power of two to pad the data
	ntpad=jpad2(nt)
	write(0,*) "pad", ntpad

	!! allocate space for complex temporary array
	allocate(ctemp(ntpad),ctemp1(ntpad),ctemp2(ntpad))

	!! main loop over traces (columns of matrix data)
	do ix=1,nx

		!! make data complex to use complex-to-complex FFT
		ctemp(1:nt)=scale*cmplx(data(:,ix),0.)
		ctemp1(1:nt)=scale*cmplx(filt(:,ix),0.)

		!! pad with zeros to next power of two
		ctemp(nt+1:ntpad)=(0.,0.)
		ctemp1(nt+1:ntpad)=(0.,0.)

		!! forward FFT transform
		call jftu(1., ntpad, ctemp)
		call jftu(1., ntpad, ctemp1)
		
		!! source average 
		ave=(1./real(ntpad))*real(sum(ctemp1*conjg(ctemp1))) 

		!! compute deconvolution 
                cont=0
                do it=1, ntpad
		   if (real(ctemp1(it)*conjg(ctemp1(it))) < eps ) then
		      ctemp2(it)=(0.,0.)
                      cont=cont+1
                   else
                       ctemp2(it)=(ctemp(it)*conjg(ctemp1(it)))/(ctemp1(it)*conjg(ctemp1(it)))
                   end if 
                write(0,*)(ctemp(it)*conjg(ctemp1(it))),real(ctemp1(it)*conjg(ctemp1(it))),ctemp2(it)
		end do
                write(0,*) "cont,ntpad-cont,sum,average",cont,ntpad-cont,sum(ctemp2),sum(ctemp2)/(ntpad-cont)

	        !! inverse FFT
		call jftu(-1., ntpad, ctemp2)
		
		!! copy result to output array discarding padding and apply factor
                !! to account for zeros in the division 
		decon(1:nt,ix)=ctemp2(1:nt)
		decon(1,ix)=(real(ntpad)/real(ntpad-cont))*(ctemp2(1))
	end do

	deallocate(ctemp,ctemp1,ctemp2)
end subroutine compute_decon



subroutine jftu (signi, nx, cx)
!   complex fast fourier transform with unitary scaling
!
!              nx          2*pi*i*(j-1)*(k-1)/nx
!   cx(k)  =  sum cx(j) * e
!             j=1             for k=1,2,...,nx=2**integer
!
!               1      nx          -2*pi*i*(j-1)*(k-1)/nx
!   cx(k)  =  ----- * sum cx(j) * e
!              (nx)   j=1             for k=1,2,...,nx=2**integer
!

	!! subroutine arguments
	real						:: signi
	integer						:: nx
	complex 					:: cx(nx)

	!! subroutine variables
	integer 					:: i, j, k, m, istep
	real*8  					:: arg,pi
	complex 					:: cw, cdel, ct

	!! check and scale input
	pi=4.0d0*datan(1.0d0)
	if(nx /= jpad2(nx))  call erexit('ftu: nx not a power of 2')
        if ( signi == -1. ) then 
!	   cx = cx/sqrt(1.*nx)
	   cx = cx/(1.*nx)
        end if
	j = 1;  k = 1

	!! compute FFT
	do i= 1, nx
		if (i<=j) then
			ct = cx(j)
			cx(j) = cx(i)
			cx(i) = ct
		end if
		m = nx/2
		do while (j>m .AND. m>1)
			j = j-m
			m = m/2
		end do
			j = j+m
	end do
	do 
		istep = 2*k
		cw = 1.
		arg = signi*pi/k
		cdel = cmplx (cos(arg), sin(arg))
		do m= 1, k
			do i= m, nx, istep
				ct=cw*cx(i+k)
				cx(i+k)=cx(i)-ct
				cx(i)=cx(i)+ct
			end do
			cw = cw * cdel
		end do
		k = istep
		if (k>=nx) exit
	end do
end subroutine jftu


integer function jpad2(n)
	integer n

	jpad2=1
	do
		if (jpad2 >= n) exit
		jpad2=jpad2*2
	end do
end function jpad2
